@@ -30,6 +30,7 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/cespare/xxhash/v2"
 	"github.com/shenwei356/bwt"
 	"github.com/twotwotwo/sorts/sortutil"
 
@@ -182,7 +183,11 @@ Examples:
 		}
 
 		// prepare pattern
-		patterns := make(map[string]*regexp.Regexp)
+		patternsR := make(map[uint64]*regexp.Regexp, 1<<10)
+		patternsN := make(map[uint64]interface{}, 1<<20)
+		// patternsS := make(map[string]interface{}, 1<<10)
+		patternsS := make([][]byte, 0, 16)
+
 		var pattern2seq *seq.Seq
 		var pbyte []byte
 		if patternFile != "" {
@@ -220,7 +225,7 @@ Examples:
 						}
 						r, err := regexp.Compile(p)
 						checkError(err)
-						patterns[p] = r
+						patternsR[xxhash.Sum64String(p)] = r
 					} else if bySeq {
 						pbyte = []byte(p)
 						if mismatches > 0 && mismatches > len(p) {
@@ -230,27 +235,29 @@ Examples:
 							seq.RNAredundant.IsValid(pbyte) == nil ||
 							seq.Protein.IsValid(pbyte) == nil { // legal sequence
 							if ignoreCase {
-								patterns[strings.ToLower(p)] = nil
+								// patternsS[strings.ToLower(p)] = struct{}{}
+								patternsS = append(patternsS, bytes.ToLower(pbyte))
 							} else {
-								patterns[p] = nil
+								// patternsS[p] = struct{}{}
+								patternsS = append(patternsS, pbyte)
 							}
 						} else {
 							checkError(fmt.Errorf("illegal DNA/RNA/Protein sequence: %s", p))
 						}
 					} else {
 						if ignoreCase {
-							patterns[strings.ToLower(p)] = nil
+							patternsN[xxhash.Sum64String(strings.ToLower(p))] = struct{}{}
 						} else {
-							patterns[p] = nil
+							patternsN[xxhash.Sum64String(p)] = struct{}{}
 						}
 					}
 				}
 			}
 			if !quiet {
-				if len(patterns) == 0 {
-					log.Warningf("%d patterns loaded from file", len(patterns))
+				if len(patternsR)+len(patternsN)+len(patternsS) == 0 {
+					log.Warningf("%d patterns loaded from file", 0)
 				} else {
-					log.Infof("%d patterns loaded from file", len(patterns))
+					log.Infof("%d patterns loaded from file", len(patternsR)+len(patternsN)+len(patternsS))
 				}
 			}
 		} else {
@@ -278,7 +285,7 @@ Examples:
 					}
 					r, err := regexp.Compile(p)
 					checkError(err)
-					patterns[p] = r
+					patternsR[xxhash.Sum64String(p)] = r
 				} else if bySeq {
 					pbyte = []byte(p)
 					if mismatches > 0 && mismatches > len(p) {
@@ -288,18 +295,20 @@ Examples:
 						seq.RNAredundant.IsValid(pbyte) == nil ||
 						seq.Protein.IsValid(pbyte) == nil { // legal sequence
 						if ignoreCase {
-							patterns[strings.ToLower(p)] = nil
+							// patternsS[strings.ToLower(p)] = struct{}{}
+							patternsS = append(patternsS, bytes.ToLower(pbyte))
 						} else {
-							patterns[p] = nil
+							// patternsS[p] = struct{}{}
+							patternsS = append(patternsS, pbyte)
 						}
 					} else {
 						checkError(fmt.Errorf("illegal DNA/RNA/Protein sequence: %s", p))
 					}
 				} else {
 					if ignoreCase {
-						patterns[strings.ToLower(p)] = nil
+						patternsN[xxhash.Sum64String(strings.ToLower(p))] = struct{}{}
 					} else {
-						patterns[p] = nil
+						patternsN[xxhash.Sum64String(p)] = struct{}{}
 					}
 				}
 			}
@@ -432,7 +441,8 @@ Examples:
 						var sequence *seq.Seq
 						var target []byte
 						var hit bool
-						var k string
+						// var k string
+						var k []byte
 
 						sfmi := fmi.NewFMIndex()
 
@@ -468,8 +478,10 @@ Examples:
 							if err != nil {
 								checkError(fmt.Errorf("fail to build FMIndex for sequence: %s", record.Name))
 							}
-							for k = range patterns {
-								hit, err = sfmi.Match([]byte(k), mismatches)
+							// for k = range patternsS {
+							for _, k = range patternsS {
+								// hit, err = sfmi.Match([]byte(k), mismatches)
+								hit, err = sfmi.Match(k, mismatches)
 								if err != nil {
 									checkError(fmt.Errorf("fail to search pattern '%s' on seq '%s': %s", k, record.Name, err))
 								}
@@ -513,9 +525,10 @@ Examples:
 		var sequence *seq.Seq
 		var target []byte
 		var ok, hit bool
-		var k string
+		// var k string
+		var k []byte
 		var re *regexp.Regexp
-		var p string
+		var h uint64
 		var strand byte
 		for _, file := range files {
 			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
@@ -587,11 +600,11 @@ Examples:
 					}
 
 					if degenerate || useRegexp {
-						for p, re = range patterns {
+						for h, re = range patternsR {
 							if re.Match(target) {
 								hit = true
 								if deleteMatched && !invertMatch {
-									delete(patterns, p)
+									delete(patternsR, h)
 								}
 								break
 							}
@@ -601,12 +614,14 @@ Examples:
 							target = bytes.ToLower(target)
 						}
 						if mismatches == 0 {
-							for k = range patterns {
-								if bytes.Contains(target, []byte(k)) {
+							// for k = range patternsS {
+							for _, k = range patternsS {
+								// if bytes.Contains(target, []byte(k)) {
+								if bytes.Contains(target, k) {
 									hit = true
-									if deleteMatched && !invertMatch {
-										delete(patterns, k)
-									}
+									// if deleteMatched && !invertMatch {
+									// 	delete(patternsS, k)
+									// }
 									break
 								}
 							}
@@ -615,28 +630,27 @@ Examples:
 							if err != nil {
 								checkError(fmt.Errorf("fail to build FMIndex for sequence: %s", record.Name))
 							}
-							for k = range patterns {
-								hit, err = sfmi.Match([]byte(k), mismatches)
+							// for k = range patternsS {
+							for _, k = range patternsS {
+								// hit, err = sfmi.Match([]byte(k), mismatches)
+								hit, err = sfmi.Match(k, mismatches)
 								if err != nil {
 									checkError(fmt.Errorf("fail to search pattern '%s' on seq '%s': %s", k, record.Name, err))
 								}
 								if hit {
-									if deleteMatched && !invertMatch {
-										delete(patterns, k)
-									}
 									break
 								}
 							}
 						}
 					} else {
-						k = string(target)
+						h = xxhash.Sum64(target)
 						if ignoreCase {
-							k = strings.ToLower(k)
+							h = xxhash.Sum64(bytes.ToLower(target))
 						}
-						if _, ok = patterns[k]; ok {
+						if _, ok = patternsN[h]; ok {
 							hit = true
 							if deleteMatched && !invertMatch {
-								delete(patterns, k)
+								delete(patternsN, h)
 							}
 						}
 					}
